#ifndef Array_H
#define Array_H

#ifdef _MSC_VER
# pragma warning(disable:4786)
# pragma warning(disable:4503) // warning: decorated name length exceeded
#endif

#include <iostream>

#include <cstdlib>
#include <cstring>
#include <cassert>

/// \brief Simple vector from pre-STL times.
template <class T> class Array {
  friend std::ostream &operator<<(std::ostream &out, Array &array) {
    for(int i = 0; i < array.m_size; ++i) out << array.m_data[i]; return out;}
  friend std::istream &operator>>(std::istream &in, Array &array) {
    for(int i = 0; i < array.m_size; ++i) in >> array.m_data[i]; return in;}
  
public:
  explicit Array(const int size) :m_size(size) {
    assert(size > 0); m_data = new T[m_size];}
  explicit Array(const int size, const T &data) :m_size(size) {
    assert(size > 0); m_data = new T[m_size];
    for(int i = 0; i < m_size; ++i) m_data[i] = data;}
  explicit Array(const int size, const T *const data) :m_size(size) {
    assert(size > 0); m_data = new T[m_size];
    for(int i = 0; i < m_size; ++i) m_data[i] = data[i];}
  Array(const Array &array) :m_size(array.m_size) {
    m_data = new T[m_size];
    for(int i = 0; i < m_size; ++i) m_data[i] = array.m_data[i];}
  virtual ~Array() {delete [] m_data;}

  T &operator[](const int number) {
    assert((number >= 0) && (number < m_size)); return m_data[number];}
  const T &operator[](const int number) const {
    assert((number >= 0) && (number < m_size)); return m_data[number];}

  T &operator()(const int number) {
    assert((number >= 0) && (number < m_size)); return m_data[number];}
  const T &operator()(const int number) const {
    assert((number >= 0) && (number < m_size)); return m_data[number];}

  operator T * () {return m_data;}
  operator const T * () const {return m_data;}

  Array<T> &operator=(const T &data) {
    for(int i = 0; i < m_size; ++i) m_data[i] = data; return *this;}
  Array<T> &operator=(const Array <T> &array) {
    assert(m_size == array.m_size);
    for(int i = 0; i < m_size; ++i) m_data[i] = array.m_data[i]; return *this;}

  Array<T> &operator=(const T *const data) {
    assert(data != NULL);
    for(int i = 0; i < m_size; ++i) m_data[i] = data[i]; return *this;}
  
  int size() const {return m_size;}

private:
  const int m_size;
  T *m_data;
};

/// Simple 2D array/'matrix'.
template <class T> class Array2d {
  friend std::ostream &operator<<(std::ostream &out, Array2d &array) {
    for(int i = 0; i < array.Size(); ++i) out << array.m_data[i]; return out;}
  friend std::istream &operator>>(std::istream &in, Array2d &array) {
    for(int i = 0; i < array.Size(); ++i) in >> array.m_data[i]; return in;}

public:
  explicit Array2d(const int rows, const int columns)
    :m_rows(rows), m_columns(columns) {
    assert(rows > 0); assert(columns > 0); m_data = new T[rows * columns];}
  explicit Array2d(const int rows, const int columns, const T &data)
    :m_rows(rows), m_columns(columns) {
    assert(rows > 0); assert(columns > 0); m_data = new T[rows * columns];
    for(int i = 0; i < m_rows * m_columns; ++i) m_data[i] = data;}
  Array2d(const Array2d &array)
    :m_rows(array.m_rows), m_columns(array.m_columns) {
    m_data = new T[m_columns * m_rows];
    for(int i = 0; i < m_rows * m_columns; ++i) m_data[i] = array.m_data[i];}
  virtual ~Array2d() {delete [] m_data;}

  T *operator[](const int row) {
    assert((row >= 0) && (row < m_rows));
    return &(m_data[row * m_columns]);}

  T &operator()(const int row, const int column) {
    assert((row >= 0) && (row < m_rows));
    assert((column >= 0) && (column < m_columns));
    return m_data[row * m_columns + column];}
  const T &operator()(const int row, const int column) const {
    assert((row >= 0) && (row < m_rows));
    assert((column >= 0) && (column < m_columns));
    return m_data[row * m_columns + column];}

  operator T * () {return m_data;}
  operator const T * () const {return m_data;}

  Array2d<T> &operator=(const T &data) {
    for(int i = 0; i < m_rows * m_columns; ++i) m_data[i] = data;
    return *this;}
  Array2d<T> &operator=(const Array2d <T> &array) {
    assert(m_rows == array.m_rows);
    assert(m_columns == array.m_columns);
    for(int i = 0; i < m_rows * m_columns; ++i) m_data[i] = array.m_data[i];
    return *this;}

  int rows() const {return m_rows;}
  int columns() const {return m_columns;}
  int size() const {return m_rows * m_columns;}

private:
  const int m_rows;
  const int m_columns;
  T *m_data;
};


#endif
