#ifndef Geometry_H
#define Geometry_H

#ifdef _MSC_VER
# pragma warning(disable:4786)
# pragma warning(disable:4503) // warning: decorated name length exceeded
#endif

#include <iostream>
#include <cmath>
#include <cassert>
#include <Array.H>

/// \brief Point in N-d vector space, implementation based on Array.
template <class T> class NdPoint : public Array <T> {
  friend std::ostream &operator<<(std::ostream &out,
				  const NdPoint <T> &point) {
    out << point[0]; for(int i = 1; i < point.dimension(); ++i)
      out << ' ' << point[i]; return out;}
  friend std::istream &operator>>(std::istream &in, NdPoint <T> &point) {
    for(int i = 0; i < point.dimension(); ++i) in >> point[i]; return in;}

public:
  explicit NdPoint(const int dimension) :Array <T> (dimension) {}
  explicit NdPoint(const int dimension, const T &init) :Array <T> (dimension, init) {}
  explicit NdPoint(const int dimension, const T *const init)
    :Array <T> (dimension, init) {}
  NdPoint(const NdPoint &point) :Array <T> (point) {}

  int dimension() const {return Array <T>::size();}  
  int n() const {return dimension();}

  NdPoint &operator=(const NdPoint &point) {
    assert(dimension() == point.dimension());
    Array <T>::operator=(point); return *this;}
  NdPoint &operator=(const T &value) {
    Array <T>::operator=(value); return *this;}

  NdPoint operator+(const NdPoint <T> &a_point) const {
    assert(dimension() == a_point.dimension()); NdPoint point(*this);
    for(int i = 0; i < dimension(); ++i) point[i] += a_point[i]; return point;}
  NdPoint operator-(const NdPoint <T> &a_point) const {
    assert(dimension() == a_point.dimension()); NdPoint point(*this);
    for(int i = 0; i < dimension(); ++i) point[i] -= a_point[i]; return point;}

  NdPoint &operator+=(const NdPoint <T> &a_point) {
    assert(dimension() == a_point.dimension());
    for(int i = 0; i < dimension(); ++i)
      Array<T>::operator()(i) += a_point[i]; return *this;}
  NdPoint &operator-=(const NdPoint <T> &a_point) {
    assert(dimension() == a_point.dimension());
    for(int i = 0; i < dimension(); ++i)
      Array<T>::operator()(i) -= a_point[i]; return *this;}

  double distance(const NdPoint &point) const {
    assert(dimension() == point.dimension()); double retval = 0.0;
    for(int i = 0; i < dimension(); ++i)
      retval += (double)((point[i] - Array<T>::operator()(i)) *
			 (point[i] - Array<T>::operator()(i)));
    return sqrt(retval);}
  double distance(const double *const point) const {
    assert(point != NULL); double retval = 0.0;
    for(int i = 0; i < dimension(); ++i)
      retval += (double)((point[i] - Array<T>::operator()(i)) *
			 (point[i] - Array<T>::operator()(i)));
    return sqrt(retval);}
};



//----------------------------------------------------------------------
// Reals
//----------------------------------------------------------------------

typedef NdPoint <double> RnPoint;

//----------------------------------------------------------------------
// Integers
//----------------------------------------------------------------------

typedef NdPoint <int> ZnPoint;

#endif
