#ifndef IFAPPQ_H
#define IFAPPQ_H

#ifdef _MSC_VER
# pragma warning(disable:4786)
# pragma warning(disable:4503) // warning: decorated name length exceeded
#endif

#include <Approximator.H>
#include <ApproximatorPoint.H>
#include <Experience.H>

#include <iostream>
#include <vector>
#include <cmath>

#include <Random.H>

/// \brief Interpolative Function-Approximation based Q-learning.
/// 
/// The algorithm allows overlapping 'clusters' (aggregated states)
/// and updates multiple Q-values for each training sample.
/// See Szepesvari,Smart: Interpolation-based Q-learning,
/// to appear in Proc. of ICML 2004, also 
/// http://www.sztaki.hu/~szcsaba/papers/szws_icml2004_rlfapp.pdf
class IFAPPQ : public Approximator {
public:
  IFAPPQ(const int dimension, const int actions, const double gain,
	 const double discount, const double threshold,
	 const double epsilon = 0.01, const double count_factor = 100,
	 const double initial = 0.0, const bool random = false);
  
  int numPoints() const {return m_data.size();}
  
  void initialize(RealRandom &generator);
  
  void learn(RnPoint &point, const int action, const double value);
  
  void learn(RnPoint &state, const int action, const double reward,
	     RnPoint &next_state, bool goalState) {
    Experience e (state, action, reward, next_state,goalState);
    
    m_cache.push_back(e);
  }

  void actualLearn(RnPoint &state, const int action, const double reward,
		   RnPoint &next_state, bool goalState) {
    learn(state, action,
	  reward + (goalState?0.0:m_discount * maxQ(next_state)));
    m_flops += 2;
  }
  
  void runCache();
  
  virtual double present(RnPoint &point, const int action);
  
  virtual void read(std::istream &in);
  virtual void write(std::ostream &out) const;
  virtual int mem_usage() const {
    return m_cache.size()*(2*dimension()+2)+m_data.size()*(dimension()+4);}
  
private:
  double m_threshold;
  double m_gain;
  double m_discount;
  double m_epsilon;
  double m_count_factor;
  
  std::vector<ApproximatorPoint> m_data;
  std::vector<Experience> m_cache;
  
  void calculateDistances();
  
  // Not allowed to use these
  IFAPPQ(const IFAPPQ &approx);
  IFAPPQ &operator=(const IFAPPQ &approx);
};


#endif
